@@ -141,7 +141,7 @@ def gridder(
                 alpha = .75, 
                 linewidth =.7
             )
-            axis.set_major_formatter(FuncFormatter(lambda val, loc: f'{val:.3g}'))
+            axis.set_major_formatter(FuncFormatter(lambda val, loc: f'{val:.3g}' if (val % 1) else int(val)))
         if mi is not None or scale == 'log': 
             if scale == 'linear':
                 locator = MultipleLocator(mi)
@@ -407,7 +407,7 @@ class Space:
             n = (stop - start) / step
             stop = (
                 start + (round(n) - 0.5) * step
-                if (((n + 0.5) % 1) - 0.5) ** 2 < 1e-20
+                if (((n + 0.5) % 1) - 0.5) ** 2 < 1e-10
                 else stop
             )
             res.append(slice(start, stop, step))
@@ -424,7 +424,7 @@ class Space:
                     f'Slice index {i} has no step and there is no field.'
                 )
             else:
-                stop = start + (self.field.shape[i]) * step
+                stop = start + (self.field.shape[i] - 0.5) * step
             res.append(slice(start, stop, step))
         return res
     @slices.setter
@@ -472,6 +472,37 @@ the given axis.'''
                     f'Field shape index {i} ({fshi}) does not equal '
                     f'space slice shape {slsh}'
                 )
+    def __add__(self, other):
+        '''Magic for adding together spaces for extending
+axes, or assigning a field to a space instance
+before performing an operation on the result.'''
+        if type(other) == type(self):
+            sp = Space(
+                (*self.slices, *other.slices,),
+                (*self.axis_names, *other.axis_names,),
+                (*self.axis_units, *other.axis_units,),
+                (*self.freq_names, *other.freq_names,),
+                (*self.freq_units, *other.freq_units,),
+                (*self.factors, *other.factors,),
+                (*self.vert_axes, *(vi + self.ndim for vi in other.vert_axes),),
+                self.cmap,
+                self.dft_norm,
+            )
+            if self.field is not None:
+                sp = sp + self.field[tuple((
+                    *(slice(None) for i in range(self.ndim)),
+                    *(None for i in range(other.ndim)),
+                ))]
+            if other.field is not None:
+                sp = sp + other.field[tuple((
+                    *(None for i in range(self.ndim)),
+                    *(slice(None) for i in range(other.ndim)),
+                ))]
+            return sp
+        elif type(other) == np.ndarray:
+            sp = self.copy()
+            sp.field = other
+            return sp
     def __getitem_tocoords(self, slices):
         '''Takes a set of __getitem__ arguments
 and does some magic on complex values. Pads
@@ -617,38 +648,28 @@ axes.'''
         yield sp
         if self.field is not None:
             yield self.field
-        
-    def __add__(self, other):
-        '''Magic for adding together spaces for extending
-axes, or assigning a field to a space instance
-before performing an operation on the result.'''
-        if type(other) == type(self):
-            sp = Space(
-                (*self.slices, *other.slices,),
-                (*self.axis_names, *other.axis_names,),
-                (*self.axis_units, *other.axis_units,),
-                (*self.freq_names, *other.freq_names,),
-                (*self.freq_units, *other.freq_units,),
-                (*self.factors, *other.factors,),
-                (*self.vert_axes, *(vi + self.ndim for vi in other.vert_axes),),
-                self.cmap,
-                self.dft_norm,
-            )
-            if self.field is None:
-                sp = sp + self.field[tuple((
-                    *(slice(None) for i in range(self.ndim)),
-                    *(None for i in range(other.ndim)),
-                ))]
-            if other.field is None:
-                sp = sp + other.field[tuple((
-                    *(None for i in range(self.ndim)),
-                    *(slice(None) for i in range(other.ndim)),
-                ))]
-            return sp
-        elif type(other) == np.ndarray:
-            sp = self.copy()
-            sp.field = other
-            return sp
+    __str__ = lambda self: 'Space: ' + ', '.join(
+        f'{name} / {unit}'
+        for name, unit
+        in zip(self.axis_names, self.axis_units)
+    )
+    __repr__ = lambda self: 'Space:\n\t' + '\n\t'.join(
+        f'{name} / {unit} '
+        + (
+            f'({freq_name} / {freq_unit}) '
+            if freq_name is not None or freq_unit is not None
+            else ''
+        )
+        + f'[{sl.start}, {sl.stop}, {sl.step}]'
+        for name, unit, freq_name, freq_unit, sl
+        in zip(
+            self.axis_names,
+            self.axis_units,
+            self.freq_names,
+            self.freq_units,
+            self.slices,
+        )
+    )
     @property
     def properties(self):
         '''Contains the defining properties of the space as a dict,
@@ -664,6 +685,36 @@ which could be used to make a new Space instance.'''
             'cmap' : self.cmap, 
             'dft_norm' : self.dft_norm,
         }
+    def change_units(self, axis, multiplyby, newunitname, newfreqname = None, inplace = False):
+        '''Change units of an axis (or multiple in a tuple).
+By default, returns a new Space and doesn't alter initial
+Space.
+Parameters:
+    axis - int, axis to adjust
+    multiplyby - numeric, value to multiply current coordinate values by
+    newunitname - str, new name of units
+    newfreqname - str, new name of frequency domain units (None by default)
+    inplace - bool, whether to alter the current instance or else to return a new instance
+Returns:
+    sp - if inplace, then sp = None; else, sp has the new units and all other properties
+        of the initial space.'''
+        if not inplace:
+            sp = self.copy()
+            sp.change_units(axis, multiplyby, newunitname, newfreqname, True)
+            return sp
+        if can_iter(axis):
+            for args in zip(
+                axis, multiplyby, newunitname, newfreqname
+            ):
+                self.change_units(*args, True)
+        self.unit_names[axis] = newunitname
+        self.freq_names[axis] = newfreqname
+        sl = self.slices[axis]
+        self._slices[axis] = slice(
+            sl.start * multiplyby,
+            sl.stop * multiplyby,
+            sl.step * multiplyby,
+        )
     def copy(self):
         sp = Space(**self.properties)
         sp._field = self._field
@@ -1076,7 +1127,33 @@ which could be used to make a new Space instance.'''
         near_ind = round(self.coord2ind(axis, coord, dft))
         val = sl.start + sl.step * near_ind
         return val
-
+    
+    def shift_coords(self, axis, shift, inplace = False):
+        '''Shift coordinates of an axis (or multiple in a tuple).
+By default, returns a new Space and doesn't alter initial
+Space.
+Parameters:
+    axis - int, axis to adjust
+    shift - value to add to start and stop of the coordinates
+    inplace - bool, whether to alter the current instance or else to return a new instance
+Returns:
+    sp - if inplace, then sp = None; else, sp has the new shift and all other properties
+        of the initial space.'''
+        if not inplace:
+            sp = self.copy()
+            sp.shift_coords(axis, shift, True)
+            return sp
+        if can_iter(axis):
+            for args in zip(
+                axis, shift
+            ):
+                self.shift_coords(*args, True)
+        sl = self.slices[axis]
+        self._slices[axis] = slice(
+            sl.start + shift,
+            sl.stop + shift,
+            sl.step,
+        )
     def show(self, field = None, title = None, axes = None, coordinates = None,
             lims = None, show = False, dft_axes = None, vert_axis = None,
             norm = None, titleas = None, origin = 'lower', vmin = None,
